@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache.h 4682 2010-04-19 11:38:52Z tfheen $
+ * $Id: cache.h 5046 2010-07-13 10:26:47Z tfheen $
  */
 
 /*
@@ -206,6 +206,7 @@ struct worker {
 #define WORKER_MAGIC		0x6391adcf
 	struct objhead		*nobjhead;
 	struct objcore		*nobjcore;
+	void			*nhashpriv;
 	struct dstat		stats;
 
 	double			lastused;
@@ -430,8 +431,9 @@ struct sess {
 	struct sessmem		*mem;
 
 	struct workreq		workreq;
-	struct acct		acct;
+	struct acct		acct_tmp;
 	struct acct		acct_req;
+	struct acct		acct_ses;
 
 #if defined(HAVE_EPOLL_CTL)
 	struct epoll_event ev;
