@@ -31,7 +31,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_acceptor.c 4605 2010-03-09 11:27:08Z phk $")
+SVNID("$Id: cache_acceptor.c 5029 2010-07-06 12:51:04Z tfheen $")
 
 #include <stdio.h>
 #include <errno.h>
@@ -177,7 +177,7 @@ VCA_Prep(struct sess *sp)
 		VSL(SLT_SessionOpen, sp->fd, "%s %s %s",
 		    sp->addr, sp->port, sp->mylsock->name);
 	}
-	sp->acct.first = sp->t_open;
+	sp->acct_ses.first = sp->t_open;
 	if (need_test)
 		sock_test(sp->fd);
 	if (need_linger)
