@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,8 +26,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_cli.c 3615 2009-02-05 11:38:45Z tfheen $
- *
  * Caching process CLI handling.
  *
  * We only have one CLI source, the stdin/stdout pipes from the manager
@@ -38,6 +36,9 @@
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_cli.c 4557 2010-02-12 11:58:43Z phk $")
+
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -49,13 +50,15 @@
 #include "cli.h"
 #include "cli_priv.h"
 #include "cli_common.h"
+#include "cli_serve.h"
 #include "cache.h"
-#include "vlu.h"
 #include "vsb.h"
 #include "hash_slinger.h"
 
 pthread_t		cli_thread;
 static struct lock	cli_mtx;
+static int		add_check;
+static struct cls	*cls;
 
 /*
  * The CLI commandlist is split in three:
@@ -65,112 +68,51 @@ static struct lock	cli_mtx;
  *  - Undocumented debug commands, show in undocumented "help -d"
  */
 
-static struct cli_proto *ccf_master_cli, *ccf_public_cli, *ccf_debug_cli;
-
 /*--------------------------------------------------------------------
  * Add CLI functions to the appropriate command set
  */
 
 void
-CLI_AddFuncs(enum cli_set_e which, struct cli_proto *p)
+CLI_AddFuncs(struct cli_proto *p)
 {
-	struct cli_proto *c, **cp;
 
-	switch (which) {
-	case MASTER_CLI: cp = &ccf_master_cli; break;
-	case PUBLIC_CLI: cp = &ccf_public_cli; break;
-	case DEBUG_CLI:	 cp = &ccf_debug_cli;  break;
-	default: INCOMPL();
-	}
+	AZ(add_check);
 	Lck_Lock(&cli_mtx);
-	c = cli_concat(*cp, p);
-	AN(c);
-	free(*cp);
-	*cp = c;
+	AZ(CLS_AddFunc(cls, 0, p));
 	Lck_Unlock(&cli_mtx);
 }
 
-/*--------------------------------------------------------------------
- * Called when we have a full line, look through all three command
- * lists to find it.
- */
-
-static int
-cli_vlu(void *priv, const char *p)
+static void
+cli_cb_before(const struct cli *cli)
 {
-	struct cli *cli;
-	int i;
 
-	cli = priv;
-	VSL(SLT_CLI, 0, "Rd %s", p);
+	VSL(SLT_CLI, 0, "Rd %s", cli->cmd);
 	VCL_Poll();
 	VBE_Poll();
-	vsb_clear(cli->sb);
 	Lck_Lock(&cli_mtx);
-	cli_dispatch(cli, ccf_master_cli, p);
-	if (cli->result == CLIS_UNKNOWN) {
-		vsb_clear(cli->sb);
-		cli->result = CLIS_OK;
-		cli_dispatch(cli, ccf_public_cli, p);
-	}
-	if (cli->result == CLIS_UNKNOWN) {
-		vsb_clear(cli->sb);
-		cli->result = CLIS_OK;
-		cli_dispatch(cli, ccf_debug_cli, p);
-	}
-	Lck_Unlock(&cli_mtx);
-	vsb_finish(cli->sb);
-	AZ(vsb_overflowed(cli->sb));
-	i = cli_writeres(heritage.cli_out, cli);
-	if (i)
-		VSL(SLT_Error, 0, "CLI write failed (errno=%d)", errno);
-	else
-		VSL(SLT_CLI, 0, "Wr %d %d %s",
-		    i, cli->result, vsb_data(cli->sb));
-	return (0);
 }
 
-/*--------------------------------------------------------------------
- * Run CLI on cli pipes from manager
- */
+static void
+cli_cb_after(const struct cli *cli)
+{
+	Lck_Unlock(&cli_mtx);
+	VSL(SLT_CLI, 0, "Wr %03u %s", cli->result, vsb_data(cli->sb));
+}
 
 void
 CLI_Run(void)
 {
-	struct pollfd pfd[1];
-	struct cli *cli, clis;
-	struct vlu *vlu;
 	int i;
 
-	cli = &clis;
-	memset(cli, 0, sizeof *cli);
+	add_check = 1;
 
-	cli->sb = vsb_newauto();
-	XXXAN(cli->sb);
-	vlu = VLU_New(cli, cli_vlu, params->cli_buffer);
-	XXXAN(vlu);
-	printf("Ready\n");
-	while (1) {
-		pfd[0].fd = heritage.cli_in;
-		pfd[0].events = POLLIN;
-		i = poll(pfd, 1, -1);
-		if (i == -1 && errno == EINTR)
-			continue;
-		assert(i == 1);
-		if (pfd[0].revents & POLLHUP) {
-			fprintf(stderr,
-			    "EOF on CLI connection, exiting\n");
-			exit(0);
-		}
-		i = VLU_Fd(heritage.cli_in, vlu);
-		if (i) {
-			fprintf(stderr,
-			    "Error on CLI connection, exiting "
-			    "(VLU_Fd %d ev: %x)\n",
-				i, pfd[0].revents);
-			break;
-		}
-	}
+	AN(CLS_AddFd(cls, heritage.cli_in, heritage.cli_out, NULL, NULL));
+
+	do {
+		i = CLS_Poll(cls, -1);
+	} while(i > 0);
+	VSL(SLT_CLI, 0, "EOF on CLI connection, worker stops");
+	VCA_Shutdown();
 }
 
 /*--------------------------------------------------------------------*/
@@ -189,48 +131,39 @@ cli_debug_sizeof(struct cli *cli, const char * const *av, void *priv)
 	SZOF(struct acct);
 	SZOF(struct worker);
 	SZOF(struct workreq);
-	SZOF(struct bereq);
 	SZOF(struct storage);
 	SZOF(struct object);
+	SZOF(struct objcore);
 	SZOF(struct objhead);
 	SZOF(struct sess);
 	SZOF(struct vbe_conn);
 	SZOF(struct varnish_stats);
+	SZOF(struct lock);
 }
 
 /*--------------------------------------------------------------------*/
 
 static void
-ccf_help(struct cli *cli, const char * const *av, void *priv)
+ccf_panic(struct cli *cli, const char * const *av, void *priv)
 {
 
+	(void)cli;
+	(void)av;
 	(void)priv;
-	cli_func_help(cli, av, ccf_public_cli);
-
-	if (av[2] != NULL && !strcmp(av[2], "-d")) {
-		/* Also list undocumented commands */
-		cli_out(cli, "\nDebugging commands:\n");
-		cli_func_help(cli, av, ccf_debug_cli);
-	} else if (cli->result == CLIS_UNKNOWN) {
-		/* Otherwise, try the undocumented list */
-		vsb_clear(cli->sb);
-		cli->result = CLIS_OK;
-		cli_func_help(cli, av, ccf_debug_cli);
-	}
+	assert(!strcmp("", "You asked for it"));
 }
 
 /*--------------------------------------------------------------------*/
 
 static struct cli_proto master_cmds[] = {
-	{ CLI_PING,		cli_func_ping },
-	{ CLI_HELP,             ccf_help, NULL },
-	{ NULL }
-};
-
-static struct cli_proto debug_cmds[] = {
+	{ CLI_PING,		"i", CLS_func_ping },
+	{ CLI_HELP,             "i", CLS_func_help },
 	{ "debug.sizeof", "debug.sizeof",
 		"\tDump sizeof various data structures\n",
-		0, 0, cli_debug_sizeof },
+		0, 0, "d", cli_debug_sizeof },
+	{ "debug.panic.worker", "debug.panic.worker",
+		"\tPanic the worker process.\n",
+		0, 0, "d", ccf_panic },
 	{ NULL }
 };
 
@@ -246,7 +179,9 @@ CLI_Init(void)
 	Lck_New(&cli_mtx);
 	cli_thread = pthread_self();
 
-	CLI_AddFuncs(MASTER_CLI, master_cmds);
-	CLI_AddFuncs(DEBUG_CLI, debug_cmds);
+	cls = CLS_New(cli_cb_before, cli_cb_after, params->cli_buffer);
+	AN(cls);
+
+	CLI_AddFuncs(master_cmds);
 }
 
