@@ -41,7 +41,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_esi.c 4678 2010-04-19 11:16:21Z tfheen $")
+SVNID("$Id: cache_esi.c 5030 2010-07-06 13:19:21Z tfheen $")
 
 #include <stdio.h>
 #include <unistd.h>
@@ -263,7 +263,7 @@ esi_addbit(struct esi_work *ew, const char *verbatim, unsigned len)
 			ew->eb->verbatim.e = TRUST_ME(verbatim + len);
 		sprintf(ew->eb->chunk_length, "%x\r\n", Tlen(ew->eb->verbatim));
 		if (params->esi_syntax & 0x4)
-			VSL(SLT_Debug, ew->sp->fd, "AddBit: %d <%.*s>",
+			WSP(ew->sp, SLT_Debug, "AddBit: %d <%.*s>",
 			    Tlen(ew->eb->verbatim),
 			    Tlen(ew->eb->verbatim),
 			    ew->eb->verbatim.b);
@@ -395,15 +395,14 @@ esi_handle_include(struct esi_work *ew)
 	txt tag;
 	txt val;
 	unsigned u, v, s;
-	struct ws *ws;
 
 	if (ew->eb == NULL || ew->eb->include.b != NULL)
 		esi_addbit(ew, NULL, 0);
 	eb = ew->eb;
-	VSL(SLT_Debug, ew->sp->fd, "Incl \"%.*s\"", t.e - t.b, t.b);
+	WSP(ew->sp, SLT_Debug, "Incl \"%.*s\"", t.e - t.b, t.b);
 	while (esi_attrib(ew, &t, &tag, &val) == 1) {
 		if (params->esi_syntax & 0x4)
-			VSL(SLT_Debug, ew->sp->fd, "<%.*s> -> <%.*s>",
+			WSP(ew->sp, SLT_Debug, "<%.*s> -> <%.*s>",
 			    tag.e - tag.b, tag.b, val.e - val.b, val.b);
 		if (Tlen(tag) != 3 || memcmp(tag.b, "src", 3))
 			continue;
@@ -413,9 +412,7 @@ esi_handle_include(struct esi_work *ew)
 			continue;
 		}
 
-		/* Wee are saving the original string */
-		ws = ew->sp->obj->ws_o;
-		WS_Assert(ws);
+		/* We are saving the original string */
 		s = 0;
 
 		if (val.b != val.e) {
@@ -626,8 +623,8 @@ parse_esi_tag(struct esi_work *ew, int closing)
 		 * XXX: Ideally, we should only pull together the bits
 		 * XXX: we need, like the filename.
 		 */
-		ew->tag.b = ew->sp->obj->ws_o->f;
-		ew->tag.e = ew->tag.b + WS_Reserve(ew->sp->obj->ws_o, 0);
+		ew->tag.b = ew->sp->wrk->ws->f;
+		ew->tag.e = ew->tag.b + WS_Reserve(ew->sp->wrk->ws, 0);
 		px = ew->s;
 		q = ew->tag.b;
 		while (px.p != ew->p.p) {
@@ -636,7 +633,7 @@ parse_esi_tag(struct esi_work *ew, int closing)
 			Nep(&px);
 		}
 		ew->tag.e = q;
-		WS_Release(ew->sp->obj->ws_o, Tlen(ew->tag));
+		WS_Release(ew->sp->wrk->ws, Tlen(ew->tag));
 	}
 	ll = Tlen(ew->tag);
 	ew->tag.b++;
@@ -651,7 +648,7 @@ parse_esi_tag(struct esi_work *ew, int closing)
 
 	ew->tag.b += 4 + (closing ? 1 : 0);
 	l = Tlen(ew->tag);
-	VSL(SLT_Debug, ew->sp->fd,
+	WSP(ew->sp, SLT_Debug,
 	    "tag {%.*s} %d %d %d", l, ew->tag.b, ew->remflg, empty, closing);
 	if (l >= 6 && !memcmp(ew->tag.b, "remove", 6)) {
 		if (empty) {
@@ -863,6 +860,7 @@ ESI_Deliver(struct sess *sp)
 	char *ws_wm;
 	struct http http_save;
 	struct esidata *ed;
+	unsigned sxid;
 
 	w = sp->wrk;
 	WRW_Reserve(w, &sp->fd);
@@ -873,7 +871,7 @@ ESI_Deliver(struct sess *sp)
 		if (Tlen(eb->verbatim)) {
 			if (sp->http->protover >= 1.1)
 				(void)WRW_Write(w, eb->chunk_length, -1);
-			sp->acct_req.bodybytes += WRW_Write(w,
+			sp->acct_tmp.bodybytes += WRW_Write(w,
 			    eb->verbatim.b, Tlen(eb->verbatim));
 			if (sp->http->protover >= 1.1)
 				(void)WRW_Write(w, "\r\n", -1);
@@ -923,6 +921,7 @@ ESI_Deliver(struct sess *sp)
 		/* Client content already taken care of */
 		http_Unset(sp->http, H_Content_Length);
 
+		sxid = sp->xid;
 		while (1) {
 			sp->wrk = w;
 			CNT_Session(sp);
@@ -933,6 +932,7 @@ ESI_Deliver(struct sess *sp)
 			DSL(0x20, SLT_Debug, sp->id, "loop waiting for ESI");
 			(void)usleep(10000);
 		}
+		sp->xid = sxid;
 		AN(sp->wrk);
 		assert(sp->step == STP_DONE);
 		sp->esis--;
