@@ -53,7 +53,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_hash.c 4703 2010-04-21 09:18:39Z tfheen $")
+SVNID("$Id: cache_hash.c 5046 2010-07-13 10:26:47Z tfheen $")
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -146,6 +146,10 @@ HSH_Cleanup(struct worker *w)
 		w->nobjhead = NULL;
 		w->stats.n_objecthead--;
 	}
+	if (w->nhashpriv != NULL) {
+		free(w->nhashpriv);
+		w->nhashpriv = NULL;
+	}
 }
 
 void
