@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_http.c 4683 2010-04-19 11:44:19Z tfheen $")
+SVNID("$Id: cache_http.c 5023 2010-07-06 11:35:23Z tfheen $")
 
 #include <errno.h>
 #include <unistd.h>
@@ -474,7 +474,7 @@ http_splitline(struct worker *w, int fd, struct http *hp,
 	q = p;
 	if (!vct_iscrlf(*p)) {
 		for (; !vct_iscrlf(*p); p++)
-			if (vct_isctl(*p))
+			if (!vct_issep(*p) && vct_isctl(*p))
 				return (-1);
 	}
 	hp->hd[h3].b = q;
