@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,12 +25,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: cache_main.c 3615 2009-02-05 11:38:45Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_main.c 4553 2010-02-11 17:43:06Z kristian $")
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -129,9 +130,15 @@ child_main(void)
 	SMS_Init();
 	STV_open();
 
-	VSL_stats->start_time = (time_t)TIM_real();
+	BAN_Compile();
+
+	/* Wait for persistent storage to load if asked to */
+	if (params->diag_bitmap & 0x00020000)
+		SMP_Ready();
 
 	CLI_Run();
 
+	STV_close();
+
 	printf("Child dies\n");
 }
