@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_pipe.c 3717 2009-02-10 14:20:29Z tfheen $
- *
  * XXX: charge bytes to srcaddr
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_pipe.c 4605 2010-03-09 11:27:08Z phk $")
+
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -67,31 +68,27 @@ PipeSession(struct sess *sp)
 {
 	struct vbe_conn *vc;
 	struct worker *w;
-	struct bereq *bereq;
 	struct pollfd fds[2];
 	int i;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
 	w = sp->wrk;
-	bereq = sp->bereq;
-	sp->bereq = NULL;
 
-	VBE_GetFd(sp);
+	sp->vbe = VBE_GetFd(NULL, sp);
 	if (sp->vbe == NULL)
 		return;
 	vc = sp->vbe;
-	TCP_blocking(vc->fd);
+	(void)TCP_blocking(vc->fd);
 
 	WRW_Reserve(w, &vc->fd);
-	sp->acct_req.hdrbytes += http_Write(w, bereq->http, 0);
+	sp->acct_req.hdrbytes += http_Write(w, sp->wrk->bereq, 0);
 
 	if (sp->htc->pipeline.b != NULL)
 		sp->acct_req.bodybytes +=
 		    WRW_Write(w, sp->htc->pipeline.b, Tlen(sp->htc->pipeline));
 
 	i = WRW_FlushRelease(w);
-	VBE_free_bereq(&bereq);
 
 	if (i) {
 		vca_close_session(sp, "pipe");
@@ -102,8 +99,12 @@ PipeSession(struct sess *sp)
 	sp->t_resp = TIM_real();
 
 	memset(fds, 0, sizeof fds);
+
+	// XXX: not yet (void)TCP_linger(vc->fd, 0);
 	fds[0].fd = vc->fd;
 	fds[0].events = POLLIN | POLLERR;
+
+	// XXX: not yet (void)TCP_linger(sp->fd, 0);
 	fds[1].fd = sp->fd;
 	fds[1].events = POLLIN | POLLERR;
 
