@@ -30,7 +30,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_response.c 4763 2010-05-05 07:48:18Z tfheen $")
+SVNID("$Id: cache_response.c 5029 2010-07-06 12:51:04Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -50,8 +50,6 @@ res_do_304(struct sess *sp)
 	char lm[64];
 	char *p;
 
-	WSP(sp, SLT_Length, "%u", 0);
-
 	http_ClrHeader(sp->wrk->resp);
 	sp->wrk->resp->logtag = HTTP_Tx;
 	http_SetResp(sp->wrk->resp, "HTTP/1.1", "304", "Not Modified");
@@ -131,7 +129,7 @@ res_do_conds(struct sess *sp)
 static void
 res_dorange(struct sess *sp, const char *r, unsigned *plow, unsigned *phigh)
 {
-	unsigned low, high;
+	unsigned low, high, has_low;
 
 	(void)sp;
 	if (strncmp(r, "bytes=", 6))
@@ -139,10 +137,11 @@ res_dorange(struct sess *sp, const char *r, unsigned *plow, unsigned *phigh)
 	r += 6;
 
 	/* The low end of range */
-	low = 0;
+	has_low = low = 0;
 	if (!vct_isdigit(*r) && *r != '-')
 		return;
 	while (vct_isdigit(*r)) {
+		has_low = 1;
 		low *= 10;
 		low += *r - '0';
 		r++;
@@ -163,13 +162,17 @@ res_dorange(struct sess *sp, const char *r, unsigned *plow, unsigned *phigh)
 			high += *r - '0';
 			r++;
 		}
+		if (!has_low) {
+			low = sp->obj->len - high;
+			high = sp->obj->len - 1;
+		}
 	} else
 		high = sp->obj->len - 1;
 	if (*r != '\0')
 		return;
 
 	if (high >= sp->obj->len)
-		high = sp->obj->len;
+		high = sp->obj->len - 1;
 
 	if (low > high)
 		return;
@@ -198,8 +201,6 @@ RES_BuildHttp(struct sess *sp)
 	if (sp->obj->response == 200 && sp->http->conds && res_do_conds(sp))
 		return;
 
-	WSP(sp, SLT_Length, "%u", sp->obj->len);
-
 	http_ClrHeader(sp->wrk->resp);
 	sp->wrk->resp->logtag = HTTP_Tx;
 	http_CopyResp(sp->wrk->resp, sp->obj->http);
@@ -258,7 +259,7 @@ RES_WriteObj(struct sess *sp)
 
 		if (sp->esis == 0)
 			/* no headers for interior ESI includes */
-			sp->acct_req.hdrbytes +=
+			sp->acct_tmp.hdrbytes +=
 			    http_Write(sp->wrk, sp->wrk->resp, 1);
 
 		if (WRW_FlushRelease(sp->wrk)) {
@@ -283,7 +284,7 @@ RES_WriteObj(struct sess *sp)
 		    http_GetHdr(sp->http, H_Range, &r))
 			res_dorange(sp, r, &low, &high);
 
-		sp->acct_req.hdrbytes += http_Write(sp->wrk, sp->wrk->resp, 1);
+		sp->acct_tmp.hdrbytes += http_Write(sp->wrk, sp->wrk->resp, 1);
 	} else if (!sp->disable_esi &&
 	    sp->esis > 0 &&
 	    sp->http->protover >= 1.1 &&
@@ -329,7 +330,7 @@ RES_WriteObj(struct sess *sp)
 
 		ptr += len;
 
-		sp->acct_req.bodybytes += len;
+		sp->acct_tmp.bodybytes += len;
 #ifdef SENDFILE_WORKS
 		/*
 		 * XXX: the overhead of setting up sendfile is not
