@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,8 +25,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_vary.c 2968 2008-07-19 12:38:28Z phk $
- *
  * Do Vary processing.
  *
  * When we insert an object into the cache which has a Vary: header,
@@ -56,22 +54,25 @@
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cache_vary.c 4440 2010-01-08 16:53:38Z phk $")
+
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "cache.h"
 
-void
-VRY_Create(const struct sess *sp)
+struct vsb *
+VRY_Create(const struct sess *sp, const struct http *hp)
 {
 	char *v, *p, *q, *h, *e;
 	struct vsb *sb, *sbh;
 	int l;
 
 	/* No Vary: header, no worries */
-	if (!http_GetHdr(sp->obj->http, H_Vary, &v))
-		return;
+	if (!http_GetHdr(hp, H_Vary, &v))
+		return (NULL);
 
 	/* For vary matching string */
 	sb = vsb_newauto();
@@ -125,16 +126,10 @@ VRY_Create(const struct sess *sp)
 	/* Terminate vary matching string */
 	vsb_printf(sb, "%c", 0);
 
+	vsb_delete(sbh);
 	vsb_finish(sb);
 	AZ(vsb_overflowed(sb));
-	l = vsb_len(sb);
-	assert(l >= 0);
-	sp->obj->vary = malloc(l);
-	AN(sp->obj->vary);
-	memcpy(sp->obj->vary, vsb_data(sb), l);
-
-	vsb_delete(sb);
-	vsb_delete(sbh);
+	return(sb);
 }
 
 int
