@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_vrt.c 4764 2010-05-05 08:02:47Z tfheen $")
+SVNID("$Id: cache_vrt.c 5047 2010-07-13 11:07:00Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -696,10 +696,12 @@ VRT_r_client_ip(const struct sess *sp)
 struct sockaddr *
 VRT_r_server_ip(struct sess *sp)
 {
+	int i;
 
-	if (sp->mysockaddr->sa_family == AF_UNSPEC)
-		assert(!getsockname(sp->fd, sp->mysockaddr, &sp->mysockaddrlen)
-		    || errno == ECONNRESET);
+	if (sp->mysockaddr->sa_family == AF_UNSPEC) {
+		i = getsockname(sp->fd, sp->mysockaddr, &sp->mysockaddrlen);
+		assert(TCP_Check(i));
+	}
 
 	return (sp->mysockaddr);
 }
@@ -937,6 +939,20 @@ VRT_synth_page(struct sess *sp, unsigned flags, const char *str, ...)
 
 /*--------------------------------------------------------------------*/
 
+void
+VRT_log(struct sess *sp, const char *str, ...)
+{
+	va_list ap;
+	char *b;
+
+	va_start(ap, str);
+	b = vrt_assemble_string(sp->http, NULL, str, ap);
+	va_end(ap);
+	WSP(sp, SLT_VCL_Log, "%s", b);
+}
+
+/*--------------------------------------------------------------------*/
+
 void
 VRT_ban(struct sess *sp, char *cmds, ...)
 {
