@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: cache_vrt_re.c 4334 2009-10-20 13:55:55Z tfheen $")
+SVNID("$Id: cache_vrt_re.c 5042 2010-07-13 09:27:27Z tfheen $")
 
 #include <sys/types.h>
 
@@ -100,7 +100,7 @@ VRT_regsub(const struct sess *sp, int all, const char *str, void *re,
 	if (str == NULL)
 		return ("");
 	t = re;
-	memset(&ovector, 0, sizeof(ovector));
+	memset(ovector, 0, sizeof(ovector));
 	i = VRE_exec(t, str, strlen(str), 0, 0, ovector, 30);
 
 	/* If it didn't match, we can return the original string */
