@@ -26,7 +26,7 @@
  " * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,\n"
  " * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n"
  " *\n"
- " * $Id: default.vcl 4467 2010-01-18 13:30:47Z phk $\n"
+ " * $Id: default.vcl 5026 2010-07-06 12:06:03Z tfheen $\n"
  " *\n"
  " * The default VCL code.\n"
  " *\n"
@@ -132,9 +132,7 @@
  "    <h3>Guru Meditation:</h3>\n"
  "    <p>XID: \"} req.xid {\"</p>\n"
  "    <hr>\n"
- "    <address>\n"
- "       <a href=\"http://www.varnish-cache.org/\">Varnish cache server</a>\n"
- "    </address>\n"
+ "    <p>Varnish cache server</p>\n"
  "  </body>\n"
  "</html>\n"
  "\"};\n"
