@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: hash_classic.c 3697 2009-02-09 12:13:38Z tfheen $
- *
  * A classic bucketed hash
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: hash_classic.c 4376 2009-11-27 11:06:32Z phk $")
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -147,11 +148,8 @@ hcl_lookup(const struct sess *sp, struct objhead *noh)
 	else
 		VTAILQ_INSERT_TAIL(&hp->head, noh, hoh_list);
 
-	noh->hoh_digest = digest;
 	noh->hoh_head = hp;
 
-	HSH_Copy(sp, noh);
-
 	Lck_Unlock(&hp->mtx);
 	return (noh);
 }
@@ -173,7 +171,7 @@ hcl_deref(struct objhead *oh)
 	if (--oh->refcnt == 0) {
 		VTAILQ_REMOVE(&hp->head, oh, hoh_list);
 		ret = 0;
-	} else 
+	} else
 		ret = 1;
 	Lck_Unlock(&hp->mtx);
 	return (ret);
