@@ -30,7 +30,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: mgt_param.c 4557 2010-02-12 11:58:43Z phk $")
+SVNID("$Id: mgt_param.c 4721 2010-04-26 07:36:36Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -548,9 +548,11 @@ static const struct parspec input_parspec[] = {
 	{ "default_grace", tweak_uint, &master.default_grace, 0, UINT_MAX,
 		"Default grace period.  We will deliver an object "
 		"this long after it has expired, provided another thread "
-		"is attempting to get a new copy.",
+		"is attempting to get a new copy.\n"
+		"Objects already cached will not be affected by changes "
+		"made until they are fetched from the backend again.\n",
 		DELAYED_EFFECT,
-		"10" "seconds" },
+		"10", "seconds" },
 	{ "sess_timeout", tweak_timeout, &master.sess_timeout, 0, 0,
 		"Idle timeout for persistent sessions. "
 		"If a HTTP request has not been received in this many "
@@ -811,6 +813,17 @@ static const struct parspec input_parspec[] = {
 		"A value of 0 disables saintmode.",
 		EXPERIMENTAL,
 		"10", "objects" },
+	{ "http_range_support", tweak_bool, &master.http_range_support, 0, 0,
+		"Enable support for HTTP Range headers.\n",
+		EXPERIMENTAL,
+		"off", "bool" },
+	{ "critbit_cooloff", tweak_timeout_double,
+		&master.critbit_cooloff, 60, 254,
+		"How long time the critbit hasher keeps deleted objheads "
+		"on the cooloff list.\n"
+		"A value of zero disables the ban lurker.",
+		EXPERIMENTAL,
+		"180.0", "s" },
 	{ NULL, NULL, NULL }
 };
 
