@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,20 +26,24 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: stevedore.h 3025 2008-07-25 15:27:04Z phk $
+ * $Id: stevedore.h 4556 2010-02-12 11:42:03Z phk $
  */
 
-#include "vqueue.h"
-
 struct stevedore;
 struct sess;
 struct iovec;
+struct object;
+struct objcore;
 
 typedef void storage_init_f(struct stevedore *, int ac, char * const *av);
 typedef void storage_open_f(const struct stevedore *);
-typedef struct storage *storage_alloc_f(struct stevedore *, size_t size);
-typedef void storage_trim_f(const struct storage *, size_t size);
+typedef struct storage *storage_alloc_f(struct stevedore *, size_t size,
+    struct objcore *);
+typedef void storage_trim_f(struct storage *, size_t size);
 typedef void storage_free_f(struct storage *);
+typedef void storage_object_f(const struct sess *sp);
+typedef void storage_close_f(const struct stevedore *);
+
 
 struct stevedore {
 	unsigned		magic;
@@ -50,6 +54,10 @@ struct stevedore {
 	storage_alloc_f		*alloc;
 	storage_trim_f		*trim;
 	storage_free_f		*free;
+	storage_object_f	*object;
+	storage_close_f		*close;
+
+	struct lru		*lru;
 
 	/* private fields */
 	void			*priv;
@@ -57,11 +65,30 @@ struct stevedore {
 	VTAILQ_ENTRY(stevedore)	list;
 };
 
-struct storage *STV_alloc(struct sess *sp, size_t size);
-void STV_trim(const struct storage *st, size_t size);
+struct object *STV_NewObject(struct sess *sp, unsigned len, double ttl,
+    unsigned nhttp);
+struct storage *STV_alloc(struct sess *sp, size_t size, struct objcore *oc);
+void STV_trim(struct storage *st, size_t size);
 void STV_free(struct storage *st);
 void STV_add(const struct stevedore *stv, int ac, char * const *av);
 void STV_open(void);
+void STV_close(void);
+struct lru *STV_lru(const struct storage *st);
+
+struct lru *LRU_Alloc(void);
+
+int STV_GetFile(const char *fn, int *fdp, const char **fnp, const char *ctx);
+uintmax_t STV_FileSize(int fd, const char *size, unsigned *granularity,
+    const char *ctx);
 
 /* Synthetic Storage */
 void SMS_Init(void);
+
+extern struct stevedore sma_stevedore;
+extern struct stevedore smf_stevedore;
+extern struct stevedore smp_stevedore;
+#ifdef HAVE_LIBUMEM
+extern struct stevedore smu_stevedore;
+#endif
+
+extern const struct choice STV_choice[];
