@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: storage_malloc.c 3603 2009-02-05 10:34:00Z tfheen $
- *
  * Storage method based on malloc(3)
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: storage_malloc.c 4267 2009-10-01 09:12:09Z phk $")
+
 #include <sys/types.h>
 
 #include <stdio.h>
@@ -52,10 +53,11 @@ struct sma {
 };
 
 static struct storage *
-sma_alloc(struct stevedore *st, size_t size)
+sma_alloc(struct stevedore *st, size_t size, struct objcore *oc)
 {
 	struct sma *sma;
 
+	(void)oc;
 	Lck_Lock(&sma_mtx);
 	VSL_stats->sma_nreq++;
 	if (VSL_stats->sma_nbytes + size > sma_max)
@@ -104,7 +106,7 @@ sma_free(struct storage *s)
 }
 
 static void
-sma_trim(const struct storage *s, size_t size)
+sma_trim(struct storage *s, size_t size)
 {
 	struct sma *sma;
 	void *p;
@@ -119,7 +121,7 @@ sma_trim(const struct storage *s, size_t size)
 		sma->sz = size;
 		Lck_Unlock(&sma_mtx);
 		sma->s.ptr = p;
-		sma->s.space = size;
+		s->space = size;
 	}
 }
 
