@@ -32,7 +32,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: varnishd.c 4610 2010-03-15 12:14:38Z phk $")
+SVNID("$Id: varnishd.c 4687 2010-04-19 12:00:29Z tfheen $")
 
 #include <sys/stat.h>
 
@@ -424,6 +424,15 @@ main(int argc, char * const *argv)
 	struct pidfh *pfh = NULL;
 	char dirname[1024];
 
+	/*
+	 * Start out by closing all unwanted file descriptors we might
+	 * have inherited from sloppy process control daemons.
+	 */
+	for (o = getdtablesize(); o > STDERR_FILENO; o--)
+		(void)close(o);
+
+	mgt_got_fd(STDERR_FILENO);
+
 	setbuf(stdout, NULL);
 	setbuf(stderr, NULL);
 
@@ -665,6 +674,8 @@ main(int argc, char * const *argv)
 	if (!d_flag && !F_flag)
 		AZ(varnish_daemon(1, 0));
 
+	VSL_MgtPid();
+
 	if (pfh != NULL && vpf_write(pfh))
 		fprintf(stderr, "NOTE: Could not write PID file\n");
 
