@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Anders Berg <andersb@vgnett.no>
@@ -27,8 +27,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: varnishncsa.c 4003 2009-03-27 12:13:31Z tfheen $
- *
  * Obtain log data from the shared memory log, order it by session ID, and
  * display it in Apache / NCSA combined log format:
  *
@@ -61,6 +59,9 @@
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: varnishncsa.c 4480 2010-01-25 12:19:22Z kristian $")
+
 #include <ctype.h>
 #include <errno.h>
 #include <signal.h>
@@ -71,9 +72,7 @@
 #include <unistd.h>
 #include <limits.h>
 
-#ifndef HAVE_DAEMON
 #include "compat/daemon.h"
-#endif
 
 #include "vsb.h"
 #include "vpf.h"
@@ -554,6 +553,13 @@ main(int argc, char *argv[])
 		case 'w':
 			w_arg = optarg;
 			break;
+		case 'b':
+			fprintf(stderr, "-b is not valid for varnishncsa\n");
+			exit(1);
+			break;
+		case 'c':
+			/* XXX: Silently ignored: it's required anyway */
+			break;
 		default:
 			if (VSL_Arg(vd, c, optarg) > 0)
 				break;
@@ -561,6 +567,8 @@ main(int argc, char *argv[])
 		}
 	}
 
+	VSL_Arg(vd, 'c', optarg);
+
 	if (VSL_OpenLog(vd, n_arg))
 		exit(1);
 
@@ -569,7 +577,7 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
-	if (D_flag && inxorcise(0, 0) == -1) {
+	if (D_flag && varnish_daemon(0, 0) == -1) {
 		perror("daemon()");
 		if (pfh != NULL)
 			vpf_remove(pfh);
