@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -27,13 +27,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: varnishstat.c 3647 2009-02-06 10:39:34Z tfheen $
- *
  * Log tailer for Varnish
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: varnishstat.c 4553 2010-02-11 17:43:06Z kristian $")
+
 #include <sys/time.h>
 
 #include <curses.h>
@@ -125,7 +126,7 @@ do_curses(struct varnish_stats *VSL_stats, int delay, const char *fields)
 		tt = tv.tv_usec * 1e-6 + tv.tv_sec;
 		lt = tt - lt;
 
-		rt = tv.tv_sec - VSL_stats->start_time;
+		rt = VSL_stats->uptime;
 		up = rt;
 
 		mvprintw(0, 0, "%*s", COLS - 1, VSL_Name());
@@ -146,7 +147,7 @@ do_curses(struct varnish_stats *VSL_stats, int delay, const char *fields)
 		mvprintw(2, 0, "Hitrate avg:   %8.4f %8.4f %8.4f", a1, a2, a3);
 
 		line = 3;
-#define MAC_STAT(n, t, f, d) \
+#define MAC_STAT(n, t, l, f, d) \
 	if ((fields == NULL || show_field( #n, fields )) && line < LINES) { \
 		ju = VSL_stats->n; \
 		if (ju == 0 && !seen.n) { \
@@ -211,6 +212,31 @@ do_curses(struct varnish_stats *VSL_stats, int delay, const char *fields)
 	}
 }
 
+static void
+do_xml(struct varnish_stats *VSL_stats, const char* fields)
+{
+	char time_stamp[20];
+	time_t now;
+
+	printf("<?xml version=\"1.0\"?>\n");
+	now = time(NULL);
+	strftime(time_stamp, 20, "%Y-%m-%dT%H:%M:%S", localtime(&now));
+	printf("<varnishstat timestamp=\"%s\">\n", time_stamp);
+#define MAC_STAT(n, t, l, f, d) \
+	do { \
+		if (fields != NULL && ! show_field( #n, fields )) break; \
+		intmax_t ju = VSL_stats->n; \
+		printf("\t<stat>\n"); \
+		printf("\t\t<name>%s</name>\n", #n); \
+		printf("\t\t<value>%ju</value>\n", ju); \
+		printf("\t\t<description>%s</description>\n", d); \
+		printf("\t</stat>\n"); \
+	} while (0);
+#include "stat_field.h"
+#undef MAC_STAT
+	printf("</varnishstat>\n");
+}
+
 static void
 do_once(struct varnish_stats *VSL_stats, const char* fields)
 {
@@ -218,17 +244,9 @@ do_once(struct varnish_stats *VSL_stats, const char* fields)
 	double up;
 
 	gettimeofday(&tv, NULL);
-	up = tv.tv_sec - VSL_stats->start_time;
+	up = VSL_stats->uptime;
 
-	do {
-		if (fields != NULL && ! show_field("uptime", fields ))
-			break;
-		printf("%-16s %12ju %12s %s\n", "uptime",
-		    (uintmax_t)(tv.tv_sec - VSL_stats->start_time),
-		    ".  ", "Child uptime");
-	} while (0);
-
-#define MAC_STAT(n, t, f, d) \
+#define MAC_STAT(n, t, l, f, d) \
 	do { \
 		if (fields != NULL && ! show_field( #n, fields )) break; \
 		intmax_t ju = VSL_stats->n; \
@@ -259,6 +277,8 @@ usage(void)
 	fprintf(stderr, FMT, "-V", "Display the version number and exit");
 	fprintf(stderr, FMT, "-w delay",
 	    "Wait delay seconds between updates.  The default is 1.");
+	fprintf(stderr, FMT, "-x",
+	    "Print statistics once as XML and exit.");
 #undef FMT
 	exit(1);
 }
@@ -269,9 +289,8 @@ list_fields(void)
 	fprintf(stderr, "Varnishstat -f option fields:\n");
 	fprintf(stderr, "Field name           Description\n");
 	fprintf(stderr, "----------           -----------\n");
-	fprintf(stderr, "uptime               Child uptime\n");
 
-#define MAC_STAT(n, t, f, d) \
+#define MAC_STAT(n, t, l, f, d) \
 	do { \
 		fprintf(stderr, "%-20s %s\n", #n, d);\
 	} while (0);
@@ -284,8 +303,7 @@ valid_fields(const char* fields)
 {
 	int i, valid_field, field_length;
 	const char *all_fields[] = {
-	  "uptime",
-#define MAC_STAT(n, t, f, d) \
+#define MAC_STAT(n, t, l, f, d) \
 	#n,
 #include "stat_field.h"
 #undef MAC_STAT
@@ -330,11 +348,11 @@ main(int argc, char **argv)
 {
 	int c;
 	struct varnish_stats *VSL_stats;
-	int delay = 1, once = 0;
+	int delay = 1, once = 0, xml = 0;
 	const char *n_arg = NULL;
 	const char *fields = NULL;
 
-	while ((c = getopt(argc, argv, "1f:ln:Vw:")) != -1) {
+	while ((c = getopt(argc, argv, "1f:ln:Vw:x")) != -1) {
 		switch (c) {
 		case '1':
 			once = 1;
@@ -354,6 +372,9 @@ main(int argc, char **argv)
 		case 'w':
 			delay = atoi(optarg);
 			break;
+		case 'x':
+			xml = 1;
+			break;
 		default:
 			usage();
 		}
@@ -367,7 +388,9 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (once)
+	if (xml)
+		do_xml(VSL_stats, fields);
+	else if (once)
 		do_once(VSL_stats, fields);
 	else
 		do_curses(VSL_stats, delay, fields);
