@@ -29,7 +29,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vtc.c 4595 2010-02-26 19:16:39Z phk $")
+SVNID("$Id: vtc.c 5051 2010-07-28 10:58:20Z tfheen $")
 
 #include <stdio.h>
 #include <string.h>
@@ -610,6 +610,7 @@ main(int argc, char * const *argv)
 	double tmax, t0, t00;
 	unsigned dur = 30;
 	const char *nmax;
+	char tmpdir[BUFSIZ];
 	char cmd[BUFSIZ];
 
 	setbuf(stdout, NULL);
@@ -644,8 +645,8 @@ main(int argc, char * const *argv)
 	init_macro();
 	init_sema();
 
-	setenv("TMPDIR", "/tmp", 0);
-	vtc_tmpdir = tempnam(NULL, "vtc");
+	bprintf(tmpdir, "/tmp/vtc.%d.%08x", getpid(), (unsigned)random());
+	vtc_tmpdir = tmpdir;
 	AN(vtc_tmpdir);
 	AZ(mkdir(vtc_tmpdir, 0700));
 	macro_def(vltop, NULL, "tmpdir", vtc_tmpdir);
@@ -677,7 +678,6 @@ main(int argc, char * const *argv)
 	if (vtc_error == 0 || vtc_verbosity == 0) {
 		bprintf(cmd, "rm -rf %s", vtc_tmpdir);
 		AZ(system(cmd));
-		free(vtc_tmpdir);
 	}
 
 	if (vtc_error)
