@@ -25,9 +25,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vtc.h 3958 2009-03-19 12:50:02Z tfheen $
+ * $Id: vtc.h 4510 2010-01-29 09:56:01Z phk $
  */
 
+#include <pthread.h>
+
 struct vsb;
 struct vtclog;
 struct cmds;
@@ -52,18 +54,27 @@ cmd_f cmd_client;
 cmd_f cmd_varnish;
 cmd_f cmd_sema;
 
-extern const char *vtc_file;
-extern char *vtc_desc;
 extern int vtc_verbosity;
+extern int vtc_error;		/* Error, bail out */
+extern int vtc_stop;		/* Abandon current test, no error */
+extern pthread_t	vtc_thread;
+extern char *vtc_tmpdir;
 
 void init_sema(void);
 
-void http_process(struct vtclog *vl, const char *spec, int sock, int client);
+void http_process(struct vtclog *vl, const char *spec, int sock, int sfd);
 
 void cmd_server_genvcl(struct vsb *vsb);
 
+void vtc_loginit(void);
+void vtc_logreset(void);
+const char *vtc_logfull(void);
 struct vtclog *vtc_logopen(const char *id);
 void vtc_logclose(struct vtclog *vl);
 void vtc_log(struct vtclog *vl, unsigned lvl, const char *fmt, ...);
 void vtc_dump(struct vtclog *vl, unsigned lvl, const char *pfx,
     const char *str);
+
+void macro_def(struct vtclog *vl, const char *instance, const char *name,
+    const char *fmt, ...);
+struct vsb *macro_expand(const char *name);
