@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2009 Linpro AS
+ * Copyright (c) 2008-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -24,12 +24,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vtc_client.c 3958 2009-03-19 12:50:02Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vtc_client.c 4588 2010-02-26 09:35:42Z phk $")
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -41,6 +42,7 @@
 
 #include "vtc.h"
 
+#include "vsb.h"
 #include "vqueue.h"
 #include "miniobj.h"
 #include "vss.h"
@@ -57,6 +59,8 @@ struct client {
 
 	char			*connect;
 
+	unsigned		repeat;
+
 	pthread_t		tp;
 };
 
@@ -72,28 +76,43 @@ client_thread(void *priv)
 {
 	struct client *c;
 	struct vtclog *vl;
-	int fd = -1;
+	int fd;
 	int i;
+	unsigned u;
+	struct vsb *vsb;
+	char *p;
 
 	CAST_OBJ_NOTNULL(c, priv, CLIENT_MAGIC);
 	AN(c->connect);
 
+	p = strdup(c->connect);
+	vsb = macro_expand(p);
+	AN(vsb);
+
 	vl = vtc_logopen(c->name);
 
-	vtc_log(vl, 2, "Started");
-	vtc_log(vl, 3, "Connect to %s", c->connect);
-	fd = VSS_open(c->connect);
-	for (i = 0; fd < 0 && i < 3; i++) {
-		sleep(1);
-		fd = VSS_open(c->connect);
+	if (c->repeat == 0)
+		c->repeat = 1;
+	if (c->repeat != 1)
+		vtc_log(vl, 2, "Started (%u iterations)", c->repeat);
+	for (u = 0; u < c->repeat; u++) {
+		vtc_log(vl, 3, "Connect to %s", vsb_data(vsb));
+		fd = VSS_open(vsb_data(vsb), 0);
+		for (i = 0; fd < 0 && i < 3; i++) {
+			(void)sleep(1);
+			fd = VSS_open(vsb_data(vsb), 0);
+		}
+		if (fd < 0)
+			vtc_log(c->vl, 0, "Failed to open %s", vsb_data(vsb));
+		assert(fd >= 0);
+		vtc_log(vl, 3, "Connected to %s fd is %d", vsb_data(vsb), fd);
+		http_process(vl, c->spec, fd, -1);
+		vtc_log(vl, 3, "Closing fd %d", fd);
+		TCP_close(&fd);
 	}
-	assert(fd >= 0);
-	vtc_log(vl, 3, "Connected to %s fd is %d", c->connect, fd);
-	http_process(vl, c->spec, fd, 1);
-	vtc_log(vl, 3, "Closing fd %d", fd);
-	TCP_close(&fd);
 	vtc_log(vl, 2, "Ending");
-
+	vsb_delete(vsb);
+	free(p);
 	return (NULL);
 }
 
@@ -115,7 +134,7 @@ client_new(const char *name)
 	if (*c->name != 'c')
 		vtc_log(c->vl, 0, "Client name must start with 'c'");
 
-	REPLACE(c->connect, "127.0.0.1:9081");
+	REPLACE(c->connect, "${v1_sock}");
 	VTAILQ_INSERT_TAIL(&clients, c, list);
 	return (c);
 }
@@ -162,10 +181,8 @@ client_wait(struct client *c)
 	CHECK_OBJ_NOTNULL(c, CLIENT_MAGIC);
 	vtc_log(c->vl, 2, "Waiting for client");
 	AZ(pthread_join(c->tp, &res));
-	if (res != NULL) {
+	if (res != NULL)
 		vtc_log(c->vl, 0, "Client returned \"%s\"", (char *)res);
-		exit (1);
-	}
 	c->tp = 0;
 }
 
@@ -217,11 +234,18 @@ cmd_client(CMD_ARGS)
 	av++;
 
 	for (; *av != NULL; av++) {
+		if (vtc_error)
+			break;
 		if (!strcmp(*av, "-connect")) {
 			REPLACE(c->connect, av[1]);
 			av++;
 			continue;
 		}
+		if (!strcmp(*av, "-repeat")) {
+			c->repeat = atoi(av[1]);
+			av++;
+			continue;
+		}
 		if (!strcmp(*av, "-start")) {
 			client_start(c);
 			continue;
@@ -234,10 +258,8 @@ cmd_client(CMD_ARGS)
 			client_run(c);
 			continue;
 		}
-		if (**av == '-') {
+		if (**av == '-')
 			vtc_log(c->vl, 0, "Unknown client argument: %s", *av);
-			exit (1);
-		}
 		REPLACE(c->spec, *av);
 	}
 }
