@@ -29,7 +29,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vtc_http.c 4635 2010-03-24 09:21:26Z phk $")
+SVNID("$Id: vtc_http.c 4761 2010-05-05 07:40:13Z tfheen $")
 
 #include <poll.h>
 #include <stdio.h>
@@ -467,6 +467,7 @@ static void
 cmd_http_rxresp(CMD_ARGS)
 {
 	struct http *hp;
+	int has_obj = 1;
 
 	(void)cmd;
 	(void)vl;
@@ -476,11 +477,17 @@ cmd_http_rxresp(CMD_ARGS)
 	av++;
 
 	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxresp spec: %s\n", *av);
+		if (!strcmp(*av, "-no_obj"))
+			has_obj = 0;
+		else
+			vtc_log(hp->vl, 0,
+			    "Unknown http rxresp spec: %s\n", *av);
 	vtc_log(hp->vl, 3, "rxresp");
 	http_rxhdr(hp);
 	http_splitheader(hp, 0);
-	if (!strcmp(hp->resp[1], "200"))
+	if (!has_obj)
+		;
+	else if (!strcmp(hp->resp[1], "200"))
 		http_swallow_body(hp, hp->resp, 1);
 	else
 		http_swallow_body(hp, hp->resp, 0);
@@ -740,6 +747,37 @@ cmd_http_chunked(CMD_ARGS)
 	http_write(hp, 4, "chunked");
 }
 
+static void
+cmd_http_chunkedlen(CMD_ARGS)
+{
+	unsigned len;
+	unsigned u, v;
+	char buf[16384];
+	struct http *hp;
+
+	(void)cmd;
+	(void)vl;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AN(av[1]);
+	AZ(av[2]);
+	vsb_clear(hp->vsb);
+
+	len = atoi(av[1]);
+
+	for (u = 0; u < sizeof buf; u++)
+		buf[u] = (u & 7) + '0';
+
+	for (u = 0; u < len; u += 16384) {
+		v = len - u;
+		if (v > sizeof buf)
+			v = sizeof buf;
+		vsb_printf(hp->vsb, "%x%s", v, nl);
+		vsb_printf(hp->vsb, "%*.*s%s", v, v, buf, nl);
+	}
+	vsb_printf(hp->vsb, "%x%s", 0, nl);
+	http_write(hp, 4, "chunked");
+}
+
 /**********************************************************************
  * set the timeout
  */
@@ -861,6 +899,7 @@ static const struct cmds http_cmds[] = {
 	{ "expect",		cmd_http_expect },
 	{ "send",		cmd_http_send },
 	{ "chunked",		cmd_http_chunked },
+	{ "chunkedlen",		cmd_http_chunkedlen },
 	{ "delay",		cmd_delay },
 	{ "sema",		cmd_sema },
 	{ "expect_close",	cmd_http_expect_close },
