@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2009 Linpro AS
+ * Copyright (c) 2008-2010 Redpill Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -24,15 +24,15 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vtc_log.c 3958 2009-03-19 12:50:02Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vtc_log.c 4578 2010-02-18 11:48:32Z phk $")
+
 #include <stdio.h>
 #include <string.h>
-#include <ctype.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -45,15 +45,48 @@
 
 #include "vtc.h"
 
-int vtc_verbosity = 3;
+int vtc_verbosity = 0;
+
+static struct vsb	*vtclog_full;
+static pthread_mutex_t	vtclog_mtx;
 
 struct vtclog {
 	unsigned	magic;
 #define VTCLOG_MAGIC	0x82731202
 	const char	*id;
 	struct vsb	*vsb;
+	pthread_mutex_t	mtx;
 };
 
+/**********************************************************************/
+
+void
+vtc_loginit()
+{
+
+	vtclog_full = vsb_newauto();
+	AN(vtclog_full);
+	AZ(pthread_mutex_init(&vtclog_mtx, NULL));
+}
+
+void
+vtc_logreset()
+{
+
+	vsb_clear(vtclog_full);
+}
+
+const char *
+vtc_logfull(void)
+{
+	vsb_finish(vtclog_full);
+	AZ(vsb_overflowed(vtclog_full));
+	return (vsb_data(vtclog_full));
+}
+
+/**********************************************************************/
+
+
 struct vtclog *
 vtc_logopen(const char *id)
 {
@@ -63,6 +96,7 @@ vtc_logopen(const char *id)
 	AN(vl);
 	vl->id = id;
 	vl->vsb = vsb_newauto();
+	AZ(pthread_mutex_init(&vl->mtx, NULL));
 	return (vl);
 }
 
@@ -72,10 +106,11 @@ vtc_logclose(struct vtclog *vl)
 
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
 	vsb_delete(vl->vsb);
+	AZ(pthread_mutex_destroy(&vl->mtx));
 	FREE_OBJ(vl);
 }
 
-static const char *lead[] = {
+static const char * const lead[] = {
 	"----",
 	"#   ",
 	"##  ",
@@ -85,15 +120,27 @@ static const char *lead[] = {
 
 #define NLEAD (sizeof(lead)/sizeof(lead[0]))
 
+static void
+vtc_log_emit(struct vtclog *vl, unsigned lvl)
+{
+	if (vtc_stop && lvl == 0)
+		return;
+	AZ(pthread_mutex_lock(&vtclog_mtx));
+	vsb_cat(vtclog_full, vsb_data(vl->vsb));
+	AZ(pthread_mutex_unlock(&vtclog_mtx));
+
+	if (lvl <= vtc_verbosity)
+		(void)fputs(vsb_data(vl->vsb), stdout);
+}
+
 //lint -e{818}
 void
 vtc_log(struct vtclog *vl, unsigned lvl, const char *fmt, ...)
 {
 
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
+	AZ(pthread_mutex_lock(&vl->mtx));
 	assert(lvl < NLEAD);
-	if (lvl > vtc_verbosity)
-		return;
 	vsb_clear(vl->vsb);
 	vsb_printf(vl->vsb, "%s %-4s ", lead[lvl], vl->id);
 	va_list ap;
@@ -103,12 +150,15 @@ vtc_log(struct vtclog *vl, unsigned lvl, const char *fmt, ...)
 	vsb_putc(vl->vsb, '\n');
 	vsb_finish(vl->vsb);
 	AZ(vsb_overflowed(vl->vsb));
-	(void)fputs(vsb_data(vl->vsb), stdout);
+
+	vtc_log_emit(vl, lvl);
+
 	vsb_clear(vl->vsb);
+	AZ(pthread_mutex_unlock(&vl->mtx));
 	if (lvl == 0) {
-		printf("---- TEST FILE: %s\n", vtc_file);
-		printf("---- TEST DESCRIPTION: %s\n", vtc_desc);
-		exit (1);
+		vtc_error = 1;
+		if (pthread_self() != vtc_thread)
+			pthread_exit(NULL);
 	}
 }
 
@@ -124,8 +174,7 @@ vtc_dump(struct vtclog *vl, unsigned lvl, const char *pfx, const char *str)
 
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
 	assert(lvl < NLEAD);
-	if (lvl > vtc_verbosity)
-		return;
+	AZ(pthread_mutex_lock(&vl->mtx));
 	vsb_clear(vl->vsb);
 	if (pfx == NULL)
 		pfx = "";
@@ -155,8 +204,14 @@ vtc_dump(struct vtclog *vl, unsigned lvl, const char *pfx, const char *str)
 		vsb_printf(vl->vsb, "\n");
 	vsb_finish(vl->vsb);
 	AZ(vsb_overflowed(vl->vsb));
-	(void)fputs(vsb_data(vl->vsb), stdout);
+
+	vtc_log_emit(vl, lvl);
+
 	vsb_clear(vl->vsb);
-	if (lvl == 0)
-		exit (1);
+	AZ(pthread_mutex_unlock(&vl->mtx));
+	if (lvl == 0) {
+		vtc_error = 1;
+		if (pthread_self() != vtc_thread)
+			pthread_exit(NULL);
+	}
 }
