@@ -29,10 +29,11 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vtc_varnish.c 4595 2010-02-26 19:16:39Z phk $")
+SVNID("$Id: vtc_varnish.c 4770 2010-05-05 08:38:16Z tfheen $")
 
 #include <stdio.h>
 
+#include <limits.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <stdlib.h>
@@ -68,6 +69,8 @@ struct varnish {
 
 	struct varnish_stats	*stats;
 
+	struct vsb		*storage;
+
 	struct vsb		*args;
 	int			fds[4];
 	pid_t			pid;
@@ -171,9 +174,15 @@ varnish_new(const char *name)
 		vtc_log(v->vl, 0, "Varnish name must start with 'v'");
 
 	v->args = vsb_newauto();
+
+	v->storage = vsb_newauto();
+	vsb_printf(v->storage, "-sfile,%s,10M", v->workdir);
+	vsb_finish(v->storage);
+
 	v->cli_fd = -1;
 	VTAILQ_INSERT_TAIL(&varnishes, v, list);
 
+
 	return (v);
 }
 
@@ -269,6 +278,7 @@ varnish_launch(struct varnish *v)
 	vsb_printf(vsb, " -S %s/_S", v->workdir);
 	vsb_printf(vsb, " -M %s:%s", abuf, pbuf);
 	vsb_printf(vsb, " -P %s/varnishd.pid", v->workdir);
+	vsb_printf(vsb, " %s", vsb_data(v->storage));
 	vsb_printf(vsb, " %s", vsb_data(v->args));
 	vsb_finish(vsb);
 	AZ(vsb_overflowed(vsb));
@@ -663,6 +673,13 @@ cmd_varnish(CMD_ARGS)
 	for (; *av != NULL; av++) {
 		if (vtc_error)
 			break;
+		if (!strcmp(*av, "-storage")) {
+			vsb_clear(v->storage);
+			vsb_cat(v->storage, av[1]);
+			vsb_finish(v->storage);
+			av++;
+			continue;
+		}
 		if (!strcmp(*av, "-arg")) {
 			AN(av[1]);
 			AZ(v->pid);
