@@ -23,7 +23,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: vsb.h 3704 2009-02-09 12:43:02Z tfheen $
+ * $Id: vsb.h 4236 2009-09-14 08:47:03Z phk $
  * $FreeBSD: src/sys/sys/vsb.h,v 1.14 2004/07/09 11:35:30 des Exp $
  */
 
@@ -36,9 +36,8 @@
 struct vsb {
 	unsigned	s_magic;
 	char		*s_buf;		/* storage buffer */
-	void		*s_unused;	/* binary compatibility. */
-	int		 s_size;	/* size of storage buffer */
-	int		 s_len;		/* current length of string */
+	size_t		 s_size;	/* size of storage buffer */
+	size_t		 s_len;		/* current length of string */
 #define	VSB_FIXEDLEN	0x00000000	/* fixed length buffer (default) */
 #define	VSB_AUTOEXTEND	0x00000001	/* automatically extend buffer */
 #define	VSB_USRFLAGMSK 0x0000ffff	/* mask of flags the user may specify */
@@ -77,7 +76,8 @@ char		*vsb_data(struct vsb *);
 int		 vsb_len(struct vsb *);
 int		 vsb_done(const struct vsb *);
 void		 vsb_delete(struct vsb *);
-void		 vsb_quote(struct vsb *s, const char *p, int how);
+void		 vsb_quote(struct vsb *s, const char *p, int len, int how);
+const char	*vsb_unquote(struct vsb *s, const char *p, int len, int how);
 #ifdef __cplusplus
 };
 #endif
