@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,21 +25,20 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: cli_common.c 3704 2009-02-09 12:43:02Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: cli_common.c 4588 2010-02-26 09:35:42Z phk $")
+
 #include <sys/types.h>
 #include <sys/uio.h>
 
-#include <ctype.h>
 #include <errno.h>
 #include <poll.h>
 #include <signal.h>
 #include <stdarg.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -73,7 +72,7 @@ void
 cli_quote(struct cli *cli, const char *s)
 {
 
-	vsb_quote(cli->sb, s, 0);
+	vsb_quote(cli->sb, s, -1, 0);
 }
 
 void
@@ -86,15 +85,6 @@ cli_result(struct cli *cli, unsigned res)
 		printf("CLI result = %u\n", res);
 }
 
-void
-cli_param(struct cli *cli)
-{
-
-	cli_result(cli, CLIS_PARAM);
-	cli_out(cli,
-	    "Parameter error, use \"help [command]\" for more info.\n");
-}
-
 int
 cli_writeres(int fd, const struct cli *cli)
 {
@@ -163,8 +153,10 @@ cli_readres(int fd, unsigned *status, char **ptr, double tmo)
 	if (i != CLI_LINE0_LEN) {
 		*status = CLIS_COMMS;
 		if (ptr != NULL)
-			*ptr = strdup("CLI communication error");
-		return (1);
+			*ptr = strdup("CLI communication error (hdr)");
+		if (i != 0)
+			return (i);
+		return (*status);
 	}
 	assert(i == CLI_LINE0_LEN);
 	assert(res[3] == ' ');
@@ -173,12 +165,14 @@ cli_readres(int fd, unsigned *status, char **ptr, double tmo)
 	j = sscanf(res, "%u %u\n", &u, &v);
 	assert(j == 2);
 	*status = u;
-	p = malloc(v + 1);
+	p = malloc(v + 1L);
 	assert(p != NULL);
 	i = read_tmo(fd, p, v + 1, tmo);
 	if (i < 0) {
 		*status = CLIS_COMMS;
 		free(p);
+		if (ptr != NULL)
+			*ptr = strdup("CLI communication error (body)");
 		return (i);
 	}
 	assert(i == v + 1);
@@ -190,16 +184,3 @@ cli_readres(int fd, unsigned *status, char **ptr, double tmo)
 		*ptr = p;
 	return (0);
 }
-
-/*--------------------------------------------------------------------*/
-
-void
-cli_func_ping(struct cli *cli, const char * const *av, void *priv)
-{
-	time_t t;
-
-	(void)priv;
-	(void)av;
-	t = time(NULL);
-	cli_out(cli, "PONG %ld 1.0", t);
-}
