@@ -30,7 +30,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: tcp.c 4770 2010-05-05 08:38:16Z tfheen $")
+SVNID("$Id: tcp.c 5053 2010-07-28 11:11:56Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -236,9 +236,11 @@ TCP_connect(int s, const struct sockaddr *name, socklen_t namelen, int msec)
 void
 TCP_close(int *s)
 {
-	assert (close(*s) == 0 ||
-	    errno == ECONNRESET ||
-	    errno == ENOTCONN);
+	int i;
+
+	i = close(*s);
+
+	assert (TCP_Check(i));
 	*s = -1;
 }
 
