@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Dag-Erling Smørgrav <des@des.no>
@@ -26,13 +26,14 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: version.c 3957 2009-03-19 11:37:59Z tfheen $
- *
  * Display a standardized version message.
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: version.c 4175 2009-08-06 10:53:17Z tfheen $")
+
 #include <stdio.h>
 
 #include "libvarnish.h"
@@ -40,8 +41,8 @@
 void
 varnish_version(const char *progname)
 {
-	fprintf(stderr, "%s (%s-%s)\n", progname,
-	    PACKAGE_TARNAME, PACKAGE_VERSION);
+	fprintf(stderr, "%s (%s-%s SVN %s)\n", progname,
+		PACKAGE_TARNAME, PACKAGE_VERSION, svn_version());
 	fprintf(stderr,
 	    "Copyright (c) 2006-2009 Linpro AS / Verdens Gang AS\n");
 }
