@@ -22,14 +22,15 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vpf.c 3586 2009-02-05 09:17:38Z tfheen $
  * Derived from:
  * $FreeBSD: head/lib/libutil/pidfile.c 184091 2008-10-20 17:41:08Z des $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vpf.c 4562 2010-02-16 21:05:08Z phk $")
+
 #include <sys/param.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -193,7 +194,7 @@ vpf_write(struct pidfh *pfh)
 		return (-1);
 	}
 
-	error = snprintf(pidstr, sizeof(pidstr), "%u", getpid());
+	error = snprintf(pidstr, sizeof(pidstr), "%ju", (uintmax_t)getpid());
 	assert(error < sizeof pidstr);
 	if (pwrite(fd, pidstr, strlen(pidstr), 0) != (ssize_t)strlen(pidstr)) {
 		error = errno;
