@@ -1,5 +1,6 @@
 /*-
- * Copyright (c) 2000 Poul-Henning Kamp and Dag-Erling Smørgrav
+ * Copyright (c) 2000-2008 Poul-Henning Kamp
+ * Copyright (c) 2000-2008 Dag-Erling Coïdan Smørgrav
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,23 +13,25 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * $Id: vsb.c 3704 2009-02-09 12:43:02Z tfheen $
- * $FreeBSD: src/sys/kern/subr_sbuf.c,v 1.30 2005/12/23 11:49:53 phk Exp $
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+__FBSDID("$FreeBSD: head/sys/kern/subr_sbuf.c 181462 2008-08-09 10:26:21Z des $");
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vsb.c 4522 2010-02-01 18:48:08Z phk $")
+
 #include <ctype.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -72,6 +75,7 @@
 static void
 _vsb_assert_integrity(const char *fun, struct vsb *s)
 {
+
 	(void)fun;
 	(void)s;
 	KASSERT(s != NULL,
@@ -87,6 +91,7 @@ _vsb_assert_integrity(const char *fun, struct vsb *s)
 static void
 _vsb_assert_state(const char *fun, struct vsb *s, int state)
 {
+
 	(void)fun;
 	(void)s;
 	(void)state;
@@ -113,7 +118,6 @@ vsb_extendsize(int size)
 		else
 			newsize += VSB_MAXEXTENDINCR;
 	}
-
 	return (newsize);
 }
 
@@ -129,9 +133,8 @@ vsb_extend(struct vsb *s, int addlen)
 
 	if (!VSB_CANEXTEND(s))
 		return (-1);
-
 	newsize = vsb_extendsize(s->s_size + addlen);
-	newbuf = (char *)SBMALLOC(newsize);
+	newbuf = SBMALLOC(newsize);
 	if (newbuf == NULL)
 		return (-1);
 	memcpy(newbuf, s->s_buf, s->s_size);
@@ -152,6 +155,7 @@ vsb_extend(struct vsb *s, int addlen)
 struct vsb *
 vsb_new(struct vsb *s, char *buf, int length, int flags)
 {
+
 	KASSERT(length >= 0,
 	    ("attempt to create an vsb of negative length (%d)", length));
 	KASSERT((flags & ~VSB_USRFLAGMSK) == 0,
@@ -159,17 +163,11 @@ vsb_new(struct vsb *s, char *buf, int length, int flags)
 
 	flags &= VSB_USRFLAGMSK;
 	if (s == NULL) {
-		s = (struct vsb *)SBMALLOC(sizeof *s);
+		s = SBMALLOC(sizeof(*s));
 		if (s == NULL)
 			return (NULL);
-		if (vsb_new(s, buf, length, flags) == NULL) {
-			free(s);
-			return (NULL);
-		}
-		VSB_SETFLAG(s, VSB_DYNSTRUCT);
-		return (s);
+		flags |= VSB_DYNSTRUCT;
 	}
-
 	memset(s, 0, sizeof *s);
 	s->s_flags = flags;
 	s->s_magic = VSB_MAGIC;
@@ -180,9 +178,12 @@ vsb_new(struct vsb *s, char *buf, int length, int flags)
 	}
 	if (flags & VSB_AUTOEXTEND)
 		s->s_size = vsb_extendsize(s->s_size);
-	s->s_buf = (char *)SBMALLOC(s->s_size);
-	if (s->s_buf == NULL)
+	s->s_buf = SBMALLOC(s->s_size);
+	if (s->s_buf == NULL) {
+		if (VSB_ISDYNSTRUCT(s))
+			SBFREE(s);
 		return (NULL);
+	}
 	VSB_SETFLAG(s, VSB_DYNAMIC);
 	return (s);
 }
@@ -193,6 +194,7 @@ vsb_new(struct vsb *s, char *buf, int length, int flags)
 void
 vsb_clear(struct vsb *s)
 {
+
 	vsb_assert_integrity(s);
 	/* don't care if it's finished or not */
 
@@ -208,6 +210,7 @@ vsb_clear(struct vsb *s)
 int
 vsb_setpos(struct vsb *s, int pos)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, 0);
 
@@ -235,7 +238,6 @@ vsb_bcat(struct vsb *s, const void *buf, size_t len)
 
 	if (VSB_HASOVERFLOWED(s))
 		return (-1);
-
 	for (; len; len--) {
 		if (!VSB_HASROOM(s) && vsb_extend(s, len) < 0)
 			break;
@@ -254,6 +256,7 @@ vsb_bcat(struct vsb *s, const void *buf, size_t len)
 int
 vsb_bcpy(struct vsb *s, const void *buf, size_t len)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, 0);
 
@@ -267,6 +270,7 @@ vsb_bcpy(struct vsb *s, const void *buf, size_t len)
 int
 vsb_cat(struct vsb *s, const char *str)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, 0);
 
@@ -291,6 +295,7 @@ vsb_cat(struct vsb *s, const char *str)
 int
 vsb_cpy(struct vsb *s, const char *str)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, 0);
 
@@ -366,18 +371,18 @@ vsb_printf(struct vsb *s, const char *fmt, ...)
 int
 vsb_putc(struct vsb *s, int c)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, 0);
 
 	if (VSB_HASOVERFLOWED(s))
 		return (-1);
-
 	if (!VSB_HASROOM(s) && vsb_extend(s, 1) < 0) {
 		VSB_SETFLAG(s, VSB_OVERFLOWED);
 		return (-1);
 	}
 	if (c != '\0')
-	    s->s_buf[s->s_len++] = (char)c;
+		s->s_buf[s->s_len++] = (char)c;
 	return (0);
 }
 
@@ -387,6 +392,7 @@ vsb_putc(struct vsb *s, int c)
 int
 vsb_trim(struct vsb *s)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, 0);
 
@@ -405,7 +411,8 @@ vsb_trim(struct vsb *s)
 int
 vsb_overflowed(const struct vsb *s)
 {
-    return VSB_HASOVERFLOWED(s);
+
+	return (VSB_HASOVERFLOWED(s));
 }
 
 /*
@@ -414,6 +421,7 @@ vsb_overflowed(const struct vsb *s)
 void
 vsb_finish(struct vsb *s)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, 0);
 
@@ -428,10 +436,11 @@ vsb_finish(struct vsb *s)
 char *
 vsb_data(struct vsb *s)
 {
+
 	vsb_assert_integrity(s);
 	vsb_assert_state(s, VSB_FINISHED);
 
-	return s->s_buf;
+	return (s->s_buf);
 }
 
 /*
@@ -440,12 +449,13 @@ vsb_data(struct vsb *s)
 int
 vsb_len(struct vsb *s)
 {
+
 	vsb_assert_integrity(s);
 	/* don't care if it's finished or not */
 
 	if (VSB_HASOVERFLOWED(s))
 		return (-1);
-	return s->s_len;
+	return (s->s_len);
 }
 
 /*
@@ -481,25 +491,27 @@ vsb_done(const struct vsb *s)
  * Quote a string
  */
 void
-vsb_quote(struct vsb *s, const char *p, int how)
+vsb_quote(struct vsb *s, const char *p, int len, int how)
 {
 	const char *q;
 	int quote = 0;
 
 	(void)how;	/* For future enhancements */
+	if (len == -1)
+		len = strlen(p);
 
-	for (q = p; *q != '\0'; q++) {
+	for (q = p; q < p + len; q++) {
 		if (!isgraph(*q) || *q == '"') {
 			quote++;
 			break;
 		}
 	}
 	if (!quote) {
-		(void)vsb_cat(s, p);
+		(void)vsb_bcat(s, p, len);
 		return;
 	}
 	(void)vsb_putc(s, '"');
-	for (q = p; *q != '\0'; q++) {
+	for (q = p; q < p + len; q++) {
 		switch (*q) {
 		case ' ':
 			(void)vsb_putc(s, *q);
@@ -522,9 +534,66 @@ vsb_quote(struct vsb *s, const char *p, int how)
 			if (isgraph(*q))
 				(void)vsb_putc(s, *q);
 			else
-				(void)vsb_printf(s, "\\%o", *q);
+				(void)vsb_printf(s, "\\%o", *q & 0xff);
 			break;
 		}
 	}
 	(void)vsb_putc(s, '"');
 }
+
+/*
+ * Unquote a string
+ */
+const char *
+vsb_unquote(struct vsb *s, const char *p, int len, int how)
+{
+	const char *q;
+	char *r;
+	unsigned long u;
+	char c;
+
+	(void)how;	/* For future enhancements */
+
+	if (len == -1)
+		len = strlen(p);
+
+	for (q = p; q < p + len; q++) {
+		if (*q != '\\') {
+			(void)vsb_bcat(s, q, 1);
+			continue;
+		}
+		if (++q >= p + len)
+			return ("Incomplete '\\'-sequence at end of string");
+
+		switch(*q) {
+		case 'n':
+			(void)vsb_bcat(s, "\n", 1);
+			continue;
+		case 'r':
+			(void)vsb_bcat(s, "\r", 1);
+			continue;
+		case 't':
+			(void)vsb_bcat(s, "\t", 1);
+			continue;
+		case '0':
+		case '1':
+		case '2':
+		case '3':
+		case '4':
+		case '5':
+		case '6':
+		case '7':
+			errno = 0;
+			u = strtoul(q, &r, 8);
+			if (errno != 0 || (u & ~0xff))
+				return ("\\ooo sequence out of range");
+			c = (char)u;
+			(void)vsb_bcat(s, &c, 1);
+			q = r - 1;
+			continue;
+		default:
+			(void)vsb_bcat(s, q, 1);
+		}
+	}
+	return (NULL);
+}
