@@ -28,6 +28,9 @@
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vsha256.c 4601 2010-03-01 11:24:57Z phk $")
+
 #include <stdint.h>
 #include <string.h>
 
@@ -111,7 +114,7 @@ be32dec_vect(uint32_t *dst, const unsigned char *src, size_t len)
 		dst[i] = mybe32dec(src + i * 4);
 }
 
-#endif 
+#endif
 
 /* Elementary functions used by SHA256 */
 #define Ch(x, y, z)	((x & (y ^ z)) ^ z)
