@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2008 Linpro AS
+ * Copyright (c) 2006-2009 Linpro AS
  * All rights reserved.
  *
  * Author: Dag-Erling Smørgrav <des@des.no>
@@ -26,12 +26,13 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: vss.c 3730 2009-02-11 07:02:43Z tfheen $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: vss.c 4588 2010-02-26 09:35:42Z phk $")
+
 #include <sys/types.h>
 #include <sys/socket.h>
 
@@ -39,6 +40,7 @@
 
 #include <errno.h>
 #include <netdb.h>
+#include <poll.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -238,9 +240,9 @@ VSS_listen(const struct vss_addr *va, int depth)
  * Return the socket.
  */
 int
-VSS_connect(const struct vss_addr *va)
+VSS_connect(const struct vss_addr *va, int nonblock)
 {
-	int sd;
+	int sd, i;
 
 	sd = socket(va->va_family, va->va_socktype, va->va_protocol);
 	if (sd < 0) {
@@ -248,12 +250,14 @@ VSS_connect(const struct vss_addr *va)
 			perror("socket()");
 		return (-1);
 	}
-	if (connect(sd, &va->va_addr.sa, va->va_addrlen) != 0) {
-		perror("connect()");
-		(void)close(sd);
-		return (-1);
-	}
-	return (sd);
+	if (nonblock)
+		(void)TCP_nonblocking(sd);
+	i = connect(sd, &va->va_addr.sa, va->va_addrlen);
+	if (i == 0 || (nonblock && errno == EINPROGRESS))
+		return (sd);
+	perror("connect()");
+	(void)close(sd);
+	return (-1);
 }
 
 /*
@@ -261,12 +265,13 @@ VSS_connect(const struct vss_addr *va)
  */
 
 int
-VSS_open(const char *str)
+VSS_open(const char *str, double tmo)
 {
 	int retval;
 	char *addr = NULL, *port = NULL;
-	int nvaddr, n;
+	int nvaddr, n, i;
 	struct vss_addr **vaddr;
+	struct pollfd pfd;
 
 	retval = VSS_parse(str, &addr, &port);
 	if (retval < 0)
@@ -278,7 +283,16 @@ VSS_open(const char *str)
 		return (-1);
 	}
 	for (n = 0; n < nvaddr; n++) {
-		retval = VSS_connect(vaddr[n]);
+		retval = VSS_connect(vaddr[n], tmo != 0.0);
+		if (retval >= 0 && tmo != 0.0) {
+			pfd.fd = retval;
+			pfd.events = POLLOUT;
+			i = poll(&pfd, 1, tmo * 1e3);
+			if (i == 0 || pfd.revents != POLLOUT) {
+				(void)close(retval);
+				retval = -1;
+			}
+		}
 		if (retval >= 0)
 			break;
 	}
