@@ -25,14 +25,15 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- *
- * $Id: daemon.c 2455 2008-02-13 13:55:39Z des $
  * Derived from:
  * $FreeBSD: src/lib/libc/gen/daemon.c,v 1.8 2007/01/09 00:27:53 imp Exp $
  */
 
 #include "config.h"
 
+#include "svnid.h"
+SVNID("$Id: daemon.c 4609 2010-03-15 09:22:44Z phk $")
+
 #ifndef HAVE_DAEMON
 
 #include <errno.h>
@@ -44,7 +45,7 @@
 #include "compat/daemon.h"
 
 int
-daemon(int nochdir, int noclose)
+varnish_daemon(int nochdir, int noclose)
 {
 	struct sigaction osa, sa;
 	int fd;
@@ -64,7 +65,11 @@ daemon(int nochdir, int noclose)
 	case 0:
 		break;
 	default:
-		exit(0);
+		/*
+		 * A fine point:  _exit(0), not exit(0), to avoid triggering
+		 * atexit(3) processing
+		 */
+		_exit(0);
 	}
 
 	newgrp = setsid();
