@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vcc_compile.h 4617 2010-03-16 10:23:15Z phk $
+ * $Id: vcc_compile.h 4680 2010-04-19 11:28:51Z tfheen $
  */
 
 #include "vqueue.h"
@@ -196,9 +196,9 @@ extern struct var vcc_vars[];
 
 /* vcc_parse.c */
 void vcc_Parse(struct tokenlist *tl);
-void vcc_RTimeVal(struct tokenlist *tl);
-void vcc_TimeVal(struct tokenlist *tl);
-void vcc_SizeVal(struct tokenlist *tl);
+void vcc_RTimeVal(struct tokenlist *tl, double *);
+void vcc_TimeVal(struct tokenlist *tl, double *);
+void vcc_SizeVal(struct tokenlist *tl, double *);
 unsigned vcc_UintVal(struct tokenlist *tl);
 double vcc_DoubleVal(struct tokenlist *tl);
 
@@ -226,6 +226,8 @@ void vcc_AddToken(struct tokenlist *tl, unsigned tok, const char *b,
 /* vcc_var.c */
 struct var *vcc_FindVar(struct tokenlist *tl, const struct token *t,
     struct var *vl);
+void vcc_VarVal(struct tokenlist *tl, const struct var *vp,
+    const struct token *vt);
 
 /* vcc_xref.c */
 void vcc_AddDef(struct tokenlist *tl, struct token *t, enum ref_type type);
@@ -242,4 +244,7 @@ int vcc_CheckUses(struct tokenlist *tl);
 #define ERRCHK(tl)      do { if ((tl)->err) return; } while (0)
 #define ErrInternal(tl) vcc__ErrInternal(tl, __func__, __LINE__)
 #define Expect(a, b) vcc__Expect(a, b, __LINE__)
-#define ExpectErr(a, b) do { vcc__Expect(a, b, __LINE__); ERRCHK(a);} while (0)
+#define ExpectErr(a, b) \
+    do { vcc__Expect(a, b, __LINE__); ERRCHK(a);} while (0)
+#define SkipToken(a, b) \
+    do { vcc__Expect(a, b, __LINE__); ERRCHK(a); vcc_NextToken(a); } while (0)
