@@ -29,7 +29,7 @@
 #include "config.h"
 
 #include "svnid.h"
-SVNID("$Id: vcc_dir_round_robin.c 4411 2009-12-16 11:41:36Z phk $")
+SVNID("$Id: vcc_dir_round_robin.c 4677 2010-04-19 11:10:02Z tfheen $")
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -67,8 +67,7 @@ vcc_ParseRoundRobinDirector(struct tokenlist *tl)
 		t_be = tl->t;
 		vcc_ResetFldSpec(fs);
 
-		ExpectErr(tl, '{');
-		vcc_NextToken(tl);
+		SkipToken(tl, '{');
 		Fc(tl, 0, "\t{");
 
 		while (tl->t->tok != '}') {	/* Member fields */
